/****************************************************************************
 *
 * Project: Olimex ARM JTAG.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: testmenu.c
 * Description: Extremely simple terminal UI for accessing test functions.
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date$
 * Revision: $Revision: 29 $
 * Id: $Id$
 *
 ****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <assert.h>

#include "testmenu.h"


/**
 * @file
 * Extremely simple terminal UI for accessing test functions.
 *
 */

static char charbuf[512];

static void print(const char *msg, const struct testmenu *menu)
{
	if(menu->puts) {
		menu->puts(msg);
	} else {
		puts(msg);
	}
}



static void print_menu(const struct testmenu *menu)
{
	int i;

	print("\r\n-----------------------------------------------------\r\n", menu);
	for(i=0; i<menu->num_entries; ++i) {
		sprintf(charbuf, "    %c) %s\r\n",
				i<=9 ? (i+'0') : (i-10)+'A',
				menu->entries[i].name);
		print(charbuf, menu);
	}
	print("Please select a test to perform: ", menu);
}


static int select_test(const struct testmenu *menu)
{
	int c;

	if(menu->getc) {
		c = menu->getc();
	} else {
		c = getchar();
	}

	if(c < 0) {
		return c;
	} else if((c>='0') && (c<='9')) {
		return c - '0';
	} else if((c>='A') && (c<='Z')) {
		return c - 'A' + 10;
	} else if((c>='a') && (c<='z')) {
		return c - 'a' + 10;
	} else if(c == '?') {
		print_menu(menu);
		return -EIO;
	} else {
		print_menu(menu);
		return -EIO;
	}
}

void testmenu_poll(void)
{
	const struct testmenu *menu = &testmenu_def;
	int test_i, stat;

	/* some sanity checks */
	assert(menu->num_entries>0 && menu->num_entries<(('9'-'0')+('Z'-'A')));
	assert(menu->entries);


	print_menu(menu);

	do {
		test_i = select_test(menu);
		//if(test_i < 0)
		//	continue;

		if(test_i < 0) {
			print("\r\nInvalid character, try again: ", menu);
		} else if(test_i >= menu->num_entries) {
			print("\r\nNo such test, try again: ", menu);
		}
	} while((test_i < 0) || (test_i >= menu->num_entries));


	print("\r\n", menu);

	stat = menu->entries[test_i].test_func();

	print("Test \"", menu);
	print(menu->entries[test_i].name, menu);
	if(stat) {
		print("\" FAILED!!!\r\n", menu);
	} else {
		print(" passed.\r\n", menu);
	}

	print("Press a key to continue...", menu);
	if(menu->getc) {
		menu->getc();
	} else {
		getchar();
	}
	print("\r\n", menu);
}

